#!/usr/bin/env python

#--------------------------------------------------------------------------------
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA    02111-1307    USA
#--------------------------------------------------------------------------------

# The icons of the "Black" icon theme are licensed under a
# Creative Commons Attribution-Share Alike 3.0 License.
#--------------------------------------------------------------------------------

# Name:                volume-control.py
# Version:             1.0
# Date:                March 2008
# Description: An applet for awn to control the computer's volume.
# Authors:        Richard "nazrat" Beyer
#                 Jeff "Jawbreaker" Hubbard
#                 Pavel Panchekha <pavpanchekha@gmail.com>
#                 Spencer Creasey <screasey@gmail.com>
#                 Markus Schmidt <lsmf0815@gmx.net>

# Listing theme directories and showing icons
import os

# Search and replace on SVG
import re, math
import xml.sax.saxutils as saxutils 
# Drawing the GUI
import gtk

# Interaction with AWN
from awn.extras import AWNLib

# To later import alsaaudio
alsaaudio = None

class VolumeApplet:
    def __init__(self, awn):

        self.awn = awn
        self.awn.settings.require()

        # Scroll to change volume
        self.awn.connect("scroll-event", self.wheel)
        self.awn.timing.register(self.setIcon, 10)

        self.__lastVolume = -1
        self.__lastTheme = ""
        
        self.awn.errors.module("alsaaudio", {"Ubuntu": "python-alsaaudio",
            "Gentoo": "dev-python/pyalsaaudio",
            "Mandriva": "python-alsaaudio"}, self.__init2)

    def __init2(self, module):
        # Store alsaaudio module
        global alsaaudio
        alsaaudio = module

        self.__readSettings()
        self.readIcon()
        self.drawMainDlog()
        self.drawPrefDlog()
        self.backend.refresh()

    def __readSettings(self):
        # Theme
        try:
            self.theme = self.awn.settings["theme"]
        except:
            self.theme = "oRainge"
            self.awn.settings["theme"] = "oRainge"

        # Backend
        self.backend = Backend(self)

        try:
            self.backend.setChannel(self.awn.settings["channel"])
        except:
            self.awn.settings["channel"] = self.backend.setChannel()

    def __getIcon(self, name):
        return self.awn.icon.file(os.path.join(self.themepath, \
            "audio-volume.svg" % name))

    def setIcon(self):
        regexElement = re.compile("<([^<]+?)(awn:exec[0-9]*=\".+?\")(.*?)>", re.DOTALL)
        regexAWN = re.compile("awn:exec[0-9]*=\".+?\"")
        volume = self.backend.getVolume()
        icon = self.icon
        #Main changes for dynamic display
        if volume > 0 and volume != self.__lastVolume or self.theme != self.__lastTheme:
            
            """search for elements with awn:exec inside"""
            oldElement = regexElement.search(icon)
            while oldElement != None:
                """get the tuple with settings from svg-element awn:exec"""
                startpos = oldElement.group(2).find("=") + 2
                execTuple = eval(oldElement.group(2)[startpos:-1])
                """set the new value and escape the string for execution"""
                value = 0.0
                execute = saxutils.unescape(execTuple[2] % float(volume))
                exec(execute)
                """if we have to replace an existing value:"""
                if execTuple[3] == 0:
                    regexValue = re.compile(\
                        execTuple[0] \
                        + "\s*=\s*\"(.*?)(" \
                        +  execTuple[1]\
                        + ")(.*?)\"")
                    """check, if value is before or after awn:exec
                    and write to oldValue"""
                    oldValue = regexValue.search(oldElement.group(1))
                    if oldValue == None:
                        oldValue = regexValue.search(oldElement.group(3))
                    """create the new value element="value"""
                    newValue = execTuple[0] \
                        + "=\"" \
                        + oldValue.group(1) \
                        + "%f" % value \
                        + oldValue.group(3) \
                        + "\""
                    """in element replace the old value with the new one"""
                    newElement = regexValue.sub(newValue, oldElement.group(), count=1)
                
                else:
                    """create the new value element="value"""
                    newValue = "\n" + execTuple[0] \
                    + "=\"" \
                    + execTuple[1] % value \
                    + "\"\n"
                    """in element add new value after awn:exec"""
                    line = regexAWN.search(oldElement.group())
                    newElement = regexAWN.sub(line.group() + newValue, \
                        oldElement.group(), count=1)
                """delete awn:exec from the element"""
                newElement = regexAWN.sub("", newElement, count=1)                   
                """in icon replace the old element with the new one"""    
                icon = regexElement.sub(newElement, icon, count=1)
                """get a new start point and hit the road"""
                c = oldElement.start()
                oldElement = regexElement.search(icon, c)
            #print icon
            loader = gtk.gdk.PixbufLoader("svg")
            loader.write(icon)
            loader.close()
            self.awn.icon.set(loader.get_pixbuf())
        elif volume != self.__lastVolume or self.theme != self.__lastTheme:
            self.awn.icon.file(os.path.join(self.themepath, "audio-volume-0.svg"))
        
        self.__lastVolume = volume
        self.__lastTheme = self.theme

    def wheel(self, widget=None, event=None):
        if event.direction == gtk.gdk.SCROLL_UP:
            self.backend.up()
        elif event.direction == gtk.gdk.SCROLL_DOWN:
            self.backend.down()

    def setDevice(self, channel):
        self.backend.setChannel(channel)
        self.backend.refresh()
        self.awn.settings["channel"] = channel
    
    def readIcon(self):
    	self.themepath = os.path.join(os.path.abspath(os.path.dirname(__file__)), \
            "Themes", self.theme)
        infile = open(os.path.join(self.themepath,"audio-volume.svg"), "r")
        self.icon = infile.read()
        infile.close()
        
    def setTheme(self, theme):
        self.__lastTheme = self.theme
        self.theme = theme
        self.readIcon()
        self.backend.refresh()
        self.awn.settings["theme"] = theme
              
    def drawMainDlog(self):
        self.main = self.awn.dialog.new("main")
        cont = gtk.HBox()
        cont.set_spacing(4)
        vCont = gtk.VBox ()
        vCont.set_spacing(4)

        self.main.volume = volume = gtk.VScale()
        volume.set_range(0, 100)
        volume.set_digits(0)
        volume.set_inverted(True)
        volume.set_value(self.backend.getVolume())

        bUp = gtk.Button ()
        bUp.set_relief(gtk.RELIEF_NONE)
        bUp.set_image(gtk.image_new_from_stock(gtk.STOCK_GO_UP, \
            gtk.ICON_SIZE_BUTTON))

        bDown = gtk.Button ()
        bDown.set_relief(gtk.RELIEF_NONE)
        bDown.set_image(gtk.image_new_from_stock(gtk.STOCK_GO_DOWN, \
            gtk.ICON_SIZE_BUTTON))

        bMute = gtk.ToggleButton()
        bMute.set_relief(gtk.RELIEF_NONE)
        bMute.set_image(gtk.image_new_from_stock(gtk.STOCK_NO, \
            gtk.ICON_SIZE_BUTTON))

        bSettings = gtk.Button("Config")
        bSettings.set_relief(gtk.RELIEF_NONE)

        label = gtk.Label("Volume:")

        vCont.add(label)
        vCont.add(bUp)
        vCont.add(bDown)
        vCont.add(bMute)
        vCont.add(bSettings)
        cont.add(vCont)
        cont.add(volume)

        self.main.add(cont)

        bUp.connect("button-press-event", lambda x, y: self.backend.up())
        bDown.connect("button-press-event", lambda x, y: self.backend.down())
        bMute.connect("button-press-event", lambda x, y: self.backend.mute())
        bSettings.connect("button-press-event", lambda x, y: self.awn.dialog.toggle("prefs"))
        volume.connect("value-changed", lambda x: self.backend.setVolume(volume.get_value()))
        volume.connect("scroll-event", self.wheel)

    def drawPrefDlog(self):
        self.prefs = self.awn.dialog.new("prefs", focus=False)
        cont = gtk.VBox()
        cont.set_spacing(4)

        # Device-Combo-Box:
        dLabel = gtk.Label("Mixer Channel:")

        device = gtk.combo_box_new_text()
        device.set_title("Mixer Channel")
        for m in alsaaudio.mixers():
            device.append_text(m)

        # Theme-Combo-Box:
        tLabel = gtk.Label("Theme:")

        theme = gtk.combo_box_new_text()
        theme.set_title("Theme")
        themes = [i for i in os.listdir(os.path.join(os.path.dirname( \
            os.path.abspath(__file__)), "Themes"))]
        for i in themes:
            theme.append_text(i)

        cont.add(dLabel)
        cont.add(device)
        cont.add(tLabel)
        cont.add(theme)

        self.prefs.add(cont)

        theme.connect("changed", lambda x: self.setTheme(theme.get_active_text()))
        device.connect("changed", lambda x: self.setDevice(device.get_active_text()))

class Backend:
    def __init__(self, parent):
        self.tmpVolume = 0
        self.parent = parent

    def setChannel(self, channel=None):
        if channel:
            self.channel = channel
        else:
            try:
                self.channel = alsaaudio.mixers()[0]
            except:
                self.channel = "PCM"

        return self.channel

    def getVolume(self):
        return alsaaudio.Mixer(self.channel).getvolume()[0]

    def setVolume(self, value):
        alsaaudio.Mixer(self.channel).setvolume(int(value))
        self.parent.awn.title.set("Volume: " + str(self.getVolume()) + "%")
        self.parent.setIcon()
        self.parent.main.volume.set_value(value)

    def refresh(self):
        self.setVolume(self.getVolume())

    def up(self):
        self.setVolume(min(100, self.getVolume() + 4))
        self.parent.setIcon()

    def down(self):
        self.setVolume(max(0, self.getVolume() - 4))
        self.parent.setIcon()

    def mute(self):
        volume = self.getVolume()
        if volume > 0:
            self.tmpVolume = volume
            self.setVolume(0)
        else:
            self.setVolume(self.tmpVolume)
        self.parent.setIcon()

if __name__ == "__main__":
    # Initiation and metadata
    applet = AWNLib.initiate({"name": "Volume Control Applet",
        "short": "volume",
        "author": "Pavel Panchekha",
        "email": "pavpanchekha@gmail.com",
        "description": "An applet to control the computer's volume",
        "type": ["Audio", "Midi"]})

    # Applet creating
    VolumeApplet(applet)

    # Applet starts
    AWNLib.start(applet)
